/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.increment = exports.clearCounters = exports.getCounters = void 0;

var _constants = require("./constants");

var GQL = _constants.types.GQL,
    HTTP = _constants.types.HTTP,
    RENDER_START = _constants.types.RENDER_START,
    UPDATE_PROPERTIES = _constants.types.UPDATE_PROPERTIES,
    DISPATCH = _constants.types.DISPATCH,
    UPDATE_STATE = _constants.types.UPDATE_STATE;
var totalGQL = 0;
var totalHTTP = 0;
var totalRenders = 0;
var totalPropChange = 0;
var totalDispatch = 0;
var totalStateUpdates = 0;

var getCounters = function getCounters() {
  return {
    totalGQL: totalGQL,
    totalHTTP: totalHTTP,
    totalRenders: totalRenders,
    totalPropChange: totalPropChange,
    totalDispatch: totalDispatch,
    totalStateUpdates: totalStateUpdates
  };
};

exports.getCounters = getCounters;

var clearCounters = function clearCounters() {
  totalGQL = 0;
  totalHTTP = 0;
  totalRenders = 0;
  totalPropChange = 0;
  totalDispatch = 0;
  totalStateUpdates = 0;
};

exports.clearCounters = clearCounters;

var increment = function increment(type) {
  switch (type) {
    case GQL:
      totalGQL++;
      break;

    case HTTP:
      totalHTTP++;
      break;

    case RENDER_START:
      totalRenders++;
      break;

    case UPDATE_PROPERTIES:
      totalPropChange++;
      break;

    case DISPATCH:
      totalDispatch++;
      break;

    case UPDATE_STATE:
      totalStateUpdates++;
      break;
  }
};

exports.increment = increment;
//# sourceMappingURL=counters.js.map