/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = defineMetric;

var _levels = _interopRequireDefault(require("./levels"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Defines a custom metric
 * @function defineMetric
 * @example
 * import {defineMetric} from '@servicenow/ui-metrics';
 *
 * export const initCustomMetrics = () => {
 *		if (shouldPersistMetrics) {
 *			setInteractionId();
 *			window.addEventListener('unload', () => {
 *				persistMetrics();
 *			});
 *
 *			defineMetric('NAVIGATE', 4);
 *			persistAtInterval(intervalDuration);
 *		}
 * };
 *
 * @param {String} name
 * @param {Number} level
 */
function defineMetric(name) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  if (_levels["default"][name]) throw new Error("@servicenow/ui-metrics ".concat(name, " is already a defined metric."));
  if (!Number.isInteger(level) || level > 5 || level < 0) throw new Error("@servicenow/ui-metrics ".concat(level, " is not valid metric level."));
  _levels["default"][name] = level;
}
//# sourceMappingURL=defineMetric.js.map