/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = mark;

var _resolveContext = _interopRequireDefault(require("./resolveContext"));

var _augment = _interopRequireDefault(require("./augment"));

var _uiConfig = require("@servicenow/ui-config");

var _uiUtils = require("@servicenow/ui-utils");

var _marks = _interopRequireDefault(require("./marks"));

var _counters = require("./counters");

var _shouldMark = _interopRequireDefault(require("./shouldMark"));

var _constants = require("./constants");

var _observers = _interopRequireDefault(require("./observers"));

var _validateMarkArguments = _interopRequireDefault(require("./validateMarkArguments"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var METRICS_OBSERVER_INTERVAL = _uiConfig.configConstants.METRICS_OBSERVER_INTERVAL,
    METRICS_USE_WINDOW_PERFORMANCE = _uiConfig.configConstants.METRICS_USE_WINDOW_PERFORMANCE,
    METRICS_MARK_PRUNING_INTERVAL = _uiConfig.configConstants.METRICS_MARK_PRUNING_INTERVAL,
    METRICS_MARK_EXPIRATION = _uiConfig.configConstants.METRICS_MARK_EXPIRATION,
    METRICS_PRUNE_MARKS = _uiConfig.configConstants.METRICS_PRUNE_MARKS;
var timeoutId;
var latest = [];
var MARK_PRUNING_INTERVAL = (0, _uiConfig.getConfigItem)(METRICS_MARK_PRUNING_INTERVAL);
var MARK_EXPIRATION = (0, _uiConfig.getConfigItem)(METRICS_MARK_EXPIRATION);
var PRUNE_MARKS = (0, _uiConfig.getConfigItem)(METRICS_PRUNE_MARKS);
var _window = window,
    performance = _window.performance;

if (!(0, _uiConfig.getConfigItem)(METRICS_USE_WINDOW_PERFORMANCE)) {
  try {
    performance = window.top.performance; // eslint-disable-line prefer-destructuring
  } catch (e) {
    /* catch if !window.top */
  }
}

var callObservers = function callObservers() {
  if (_observers["default"].size) {
    clearTimeout(timeoutId);
    timeoutId = (0, _uiUtils.defer)(function () {
      (0, _uiUtils.whenIdle)(function () {
        var latestSlice = latest.splice(0);
        if (latestSlice.length) _observers["default"].forEach(function (observer) {
          return observer(_toConsumableArray(latestSlice));
        });
      });
    }, (0, _uiConfig.getConfigItem)(METRICS_OBSERVER_INTERVAL));
  }
};

var unresolvedResourceMarks = [];

var isMatch = function isMatch(mark, resource) {
  var _mark$options;

  if (isMarkBatchedRequest(mark) && isResourceBatchApi(resource)) return true;
  if (resource.name.includes(mark === null || mark === void 0 ? void 0 : (_mark$options = mark.options) === null || _mark$options === void 0 ? void 0 : _mark$options.path)) return true;
  return false;
};

var isResourceBatchApi = function isResourceBatchApi(resource) {
  return resource.name.includes('/api/now/v1/batch');
};

var isMarkBatchedRequest = function isMarkBatchedRequest(mark) {
  var _mark$options2;

  return mark === null || mark === void 0 ? void 0 : (_mark$options2 = mark.options) === null || _mark$options2 === void 0 ? void 0 : _mark$options2.batch;
}; // not currently supported in component unit testing environment


if (window.PerformanceObserver) {
  var observer = new PerformanceObserver(function (list) {
    var resources = list.getEntries().sort(function (a, b) {
      if (a.startTime < b.startTime) return -1;
      if (a.startTime > b.startTime) return 1;
      return 0;
    });

    for (var m = 0; m < unresolvedResourceMarks.length; m++) {
      var _mark = unresolvedResourceMarks[m];

      for (var e = 0; e < resources.length; e++) {
        var resource = resources[e];

        if (resource.startTime > _mark.markTime && isMatch(_mark, resource)) {
          _mark.options.resource = resource.toJSON();
          unresolvedResourceMarks.splice(m, 1);
          m -= 1;
          latest.push(_mark);
          callObservers();
          if (!isMarkBatchedRequest(_mark)) resources.splice(e, 1);
          break;
        }
      }
    }
  });
  observer.observe({
    entryTypes: ['resource']
  });
}

if (PRUNE_MARKS && MARK_PRUNING_INTERVAL > 0) {
  setInterval(function () {
    var now = performance.now();

    for (var i = 0; i < unresolvedResourceMarks.length; i++) {
      if (now - unresolvedResourceMarks[i].markTime >= MARK_EXPIRATION) {
        if (process.env.NODE_ENV !== 'production') {
          var count = unresolvedResourceMarks.length - i;
          console.warn("@servicenow/ui-metrics Pruning ".concat(count, " unresolved resource mark").concat(count > 1 ? 's' : '', ".")); // eslint-disable-line no-console
        }

        unresolvedResourceMarks.splice(i);
        break;
      }
    }

    for (var _i = 0; _i < _marks["default"].length; _i++) {
      if (now - _marks["default"][_i].markTime >= MARK_EXPIRATION) {
        if (process.env.NODE_ENV !== 'production') {
          var _count = _marks["default"].length - _i;

          console.info("@servicenow/ui-metrics Pruning ".concat(_marks["default"].length - _i, " expired mark").concat(_count > 1 ? 's' : '', ".")); // eslint-disable-line no-console
        }

        _marks["default"].splice(_i);

        break;
      }
    }
  }, MARK_PRUNING_INTERVAL);
}
/**
 * Marks a Seismic friendly performance marker
 * @function mark
 * @example
 * import {mark} from '@servicenow/ui-metrics';
 *
 * actionHandlers: {
 * 		[COMPONENT_TREE_RENDERED]({host, action: {meta}}) {
 *			if (hasFetched && !loading && !interactive) {
 *
 * 				// Mark a custom defined metric and flip interactive
 * 				// Initial interactive state is false
 *				mark(host, getInteractionId(meta), metrics.HERO_INTERACTIVE);
 *				updateState({
 *					shouldRender: false,
 *					path: 'interactive',
 *					value: true,
 *					operation: 'set'
 *				});
 *			}
 *		}
 * }
 *
 * @param {Object} context - (Optional) Context object to attach mark to
 * @param {String} interactionId - The id of the action that trigger the mark. Helps track client transactions during the interaction
 * @param {String} event - The metric type you are marking (custom metrics must be defined first).
 * @param {Object} options - (Optional) Typically this is the label or type of the mark.  Labels should be `:` delimited.
 * @param {String} type - (Optional) The category of metric.  Defaults to `performance`.  Other common options are `usage` || `both`.
 */


function mark() {
  var _augmentedMark$option, _augmentedMark$option2;

  var context = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var interactionId = arguments.length > 1 ? arguments[1] : undefined;
  var event = arguments.length > 2 ? arguments[2] : undefined;
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var type = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'performance';
  (0, _counters.increment)(event);
  var resolvedContext = (0, _resolveContext["default"])(context);
  if (!(0, _shouldMark["default"])(resolvedContext, event)) return;
  (0, _validateMarkArguments["default"])(resolvedContext, interactionId, event, options, type);
  var componentName = resolvedContext.componentName,
      componentId = resolvedContext.componentId,
      parentComponentId = resolvedContext.parentComponentId;
  var mark = {
    componentName: componentName,
    componentId: componentId,
    parentComponentId: parentComponentId,
    markTime: performance.now(),
    interactionId: interactionId,
    event: event,
    options: options,
    type: type
  };
  var augmentedMark = (0, _augment["default"])(mark);

  if (event === _constants.types.HTTP || event === _constants.types.GQL) {
    unresolvedResourceMarks.push(augmentedMark);
    augmentedMark.options.resource = {
      unresolved: true
    };
  }

  _marks["default"].push(augmentedMark);

  if (_observers["default"].size && !(augmentedMark === null || augmentedMark === void 0 ? void 0 : (_augmentedMark$option = augmentedMark.options) === null || _augmentedMark$option === void 0 ? void 0 : (_augmentedMark$option2 = _augmentedMark$option.resource) === null || _augmentedMark$option2 === void 0 ? void 0 : _augmentedMark$option2.unresolved)) {
    latest.push(augmentedMark);
    callObservers();
  }
}
//# sourceMappingURL=mark.js.map