/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = shouldMark;

var _levels = _interopRequireDefault(require("./levels"));

var _uiConfig = require("@servicenow/ui-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var METRICS_LEVEL = _uiConfig.configConstants.METRICS_LEVEL,
    METRICS_TYPES = _uiConfig.configConstants.METRICS_TYPES,
    TELEMETRY_MARKING_ENABLED = _uiConfig.configConstants.TELEMETRY_MARKING_ENABLED,
    METRICS_ELEMENT_CONFIG_ENABLED = _uiConfig.configConstants.METRICS_ELEMENT_CONFIG_ENABLED;
var globalMetricsLevel = parseInt((0, _uiConfig.getConfigItem)(METRICS_LEVEL));
var globalMetricsLevelIsInteger = Number.isInteger(globalMetricsLevel);
var globalMetricTypes = (0, _uiConfig.getConfigItem)(METRICS_TYPES);
var globalIsTelemetryEnabled = (0, _uiConfig.getConfigItem)(TELEMETRY_MARKING_ENABLED);
var globalElementMetricsConfigEnabled = (0, _uiConfig.getConfigItem)(METRICS_ELEMENT_CONFIG_ENABLED);
var resolvedLevels = {};

var isMetricTypeValid = function isMetricTypeValid(level) {
  if (resolvedLevels[level]) return true;
  var isValid = Number.isInteger(level) && level >= 0 && level <= 5;
  if (isValid) resolvedLevels[level] = true;
  return isValid;
};

var observer = function observer(path, value) {
  resolvedLevels = {};

  switch (path) {
    case METRICS_LEVEL:
      globalMetricsLevel = value;
      globalMetricsLevelIsInteger = Number.isInteger(globalMetricsLevel);
      break;

    case METRICS_TYPES:
      globalMetricTypes = value;
      break;

    case TELEMETRY_MARKING_ENABLED:
      globalIsTelemetryEnabled = value;
      break;

    case METRICS_ELEMENT_CONFIG_ENABLED:
      globalElementMetricsConfigEnabled = value;
      break;
  }
};

(0, _uiConfig.observeConfigItem)(METRICS_LEVEL, observer);
(0, _uiConfig.observeConfigItem)(METRICS_TYPES, observer);
(0, _uiConfig.observeConfigItem)(TELEMETRY_MARKING_ENABLED, observer);
(0, _uiConfig.observeConfigItem)(METRICS_ELEMENT_CONFIG_ENABLED, observer);

function shouldMark(context, metricType) {
  if (globalIsTelemetryEnabled === false) return false; // non-bootstrapped components don't have enough
  // data to send a valid mark

  if (!context.ready) return false;
  if (globalElementMetricsConfigEnabled && Array.isArray(context.nowMetricTypes)) return context.nowMetricTypes.includes(metricType);
  if (Array.isArray(globalMetricTypes)) return globalMetricTypes.includes(metricType);
  var level = _levels["default"][metricType];
  if (!isMetricTypeValid(level)) return false;
  if (globalElementMetricsConfigEnabled && Number.isInteger(context.nowMetricsLevel)) return level >= context.nowMetricsLevel;
  return globalMetricsLevelIsInteger && level >= globalMetricsLevel;
}
//# sourceMappingURL=shouldMark.js.map