/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCountersHistogram = getCountersHistogram;
exports.logTimingMetrics = exports.clearTimingMetrics = void 0;

var _constants = require("./constants");

var _uiUtils = require("@servicenow/ui-utils");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var debounceMetrics = {}; //Have to do this to avoid circular dependency between utils and metrics packages

var deferMetrics = (0, _uiUtils.getDeferMetrics)();

var clearTimingMetrics = function clearTimingMetrics() {
  debounceMetrics = {};
  (0, _uiUtils.clearDeferMetrics)();
  deferMetrics = (0, _uiUtils.getDeferMetrics)();
};

exports.clearTimingMetrics = clearTimingMetrics;

var logTimingMetrics = function logTimingMetrics(type) {
  var duration = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var componentName = arguments.length > 2 ? arguments[2] : undefined;

  switch (type) {
    case _constants.timingMetricTypes.DEFER:
      (0, _uiUtils.logDuration)(deferMetrics, duration, componentName);
      break;

    case _constants.timingMetricTypes.DEBOUNCE:
      (0, _uiUtils.logDuration)(debounceMetrics, duration, componentName);
      break;
  }
};

exports.logTimingMetrics = logTimingMetrics;

function getCountersHistogram() {
  var _ref;

  return _ref = {}, _defineProperty(_ref, _constants.timingMetricTypes.DEFER, getHistogram(deferMetrics)), _defineProperty(_ref, _constants.timingMetricTypes.DEBOUNCE, getHistogram(debounceMetrics)), _ref;
}

function getHistogram(metrics) {
  var _histogram;

  var histogram = (_histogram = {}, _defineProperty(_histogram, _constants.timings.LOWER_BOUNDARY, {}), _defineProperty(_histogram, _constants.timings.MIDDLE, {}), _defineProperty(_histogram, _constants.timings.UPPER_BOUNDARY, {}), _histogram);
  Object.keys(metrics).forEach(function (componentName) {
    var delayTimeArray = metrics[componentName] || [];
    delayTimeArray.forEach(function (delayTime) {
      if (delayTime <= 50) {
        histogram[_constants.timings.LOWER_BOUNDARY][componentName] = (histogram[_constants.timings.LOWER_BOUNDARY][componentName] ? histogram[_constants.timings.LOWER_BOUNDARY][componentName] : 0) + 1;
      } else if (delayTime > 100) {
        histogram[_constants.timings.UPPER_BOUNDARY][componentName] = (histogram[_constants.timings.UPPER_BOUNDARY][componentName] ? histogram[_constants.timings.UPPER_BOUNDARY][componentName] : 0) + 1;
      } else {
        histogram[_constants.timings.MIDDLE][componentName] = (histogram[_constants.timings.MIDDLE][componentName] ? histogram[_constants.timings.MIDDLE][componentName] : 0) + 1;
      }
    });
  });
  return histogram;
}
//# sourceMappingURL=timingMetrics.js.map