/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import levels from './levels';
/**
 * Defines a custom metric
 * @function defineMetric
 * @example
 * import {defineMetric} from '@servicenow/ui-metrics';
 *
 * export const initCustomMetrics = () => {
 *		if (shouldPersistMetrics) {
 *			setInteractionId();
 *			window.addEventListener('unload', () => {
 *				persistMetrics();
 *			});
 *
 *			defineMetric('NAVIGATE', 4);
 *			persistAtInterval(intervalDuration);
 *		}
 * };
 *
 * @param {String} name
 * @param {Number} level
 */

export default function defineMetric(name, level = 0) {
  if (levels[name]) throw new Error(`@servicenow/ui-metrics ${name} is already a defined metric.`);
  if (!Number.isInteger(level) || level > 5 || level < 0) throw new Error(`@servicenow/ui-metrics ${level} is not valid metric level.`);
  levels[name] = level;
}
//# sourceMappingURL=defineMetric.js.map