/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { INTERACTION_ID } from './symbols';
import validateMeta from './validateMeta';
/**
 * Retrieves the interactionId from an action
 * @function getInteractionId
 * @example
 * import {getInteractionId} from '@servicenow/ui-metrics';
 *
 * const captureClickEvent = (coeffects, target) => {
 * 	 const {action, host} = coeffects;
 *
 *   mark(host, getInteractionId(action.meta), 'CLICK_EVENT', {eventTarget: target});
 * }
 *
 * @param {Object} meta - The meta data from the action object
 */

export default function getInteractionId(meta) {
  validateMeta(meta, 'getInteractionId');
  return meta[INTERACTION_ID];
}
//# sourceMappingURL=getInteractionId.js.map