/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import levels from './levels';
import { getConfigItem, configConstants, observeConfigItem } from '@servicenow/ui-config';
const {
  METRICS_LEVEL,
  METRICS_TYPES,
  TELEMETRY_MARKING_ENABLED,
  METRICS_ELEMENT_CONFIG_ENABLED
} = configConstants;
let globalMetricsLevel = parseInt(getConfigItem(METRICS_LEVEL));
let globalMetricsLevelIsInteger = Number.isInteger(globalMetricsLevel);
let globalMetricTypes = getConfigItem(METRICS_TYPES);
let globalIsTelemetryEnabled = getConfigItem(TELEMETRY_MARKING_ENABLED);
let globalElementMetricsConfigEnabled = getConfigItem(METRICS_ELEMENT_CONFIG_ENABLED);
let resolvedLevels = {};

const isMetricTypeValid = level => {
  if (resolvedLevels[level]) return true;
  const isValid = Number.isInteger(level) && level >= 0 && level <= 5;
  if (isValid) resolvedLevels[level] = true;
  return isValid;
};

const observer = (path, value) => {
  resolvedLevels = {};

  switch (path) {
    case METRICS_LEVEL:
      globalMetricsLevel = value;
      globalMetricsLevelIsInteger = Number.isInteger(globalMetricsLevel);
      break;

    case METRICS_TYPES:
      globalMetricTypes = value;
      break;

    case TELEMETRY_MARKING_ENABLED:
      globalIsTelemetryEnabled = value;
      break;

    case METRICS_ELEMENT_CONFIG_ENABLED:
      globalElementMetricsConfigEnabled = value;
      break;
  }
};

observeConfigItem(METRICS_LEVEL, observer);
observeConfigItem(METRICS_TYPES, observer);
observeConfigItem(TELEMETRY_MARKING_ENABLED, observer);
observeConfigItem(METRICS_ELEMENT_CONFIG_ENABLED, observer);
export default function shouldMark(context, metricType) {
  if (globalIsTelemetryEnabled === false) return false; // non-bootstrapped components don't have enough
  // data to send a valid mark

  if (!context.ready) return false;
  if (globalElementMetricsConfigEnabled && Array.isArray(context.nowMetricTypes)) return context.nowMetricTypes.includes(metricType);
  if (Array.isArray(globalMetricTypes)) return globalMetricTypes.includes(metricType);
  const level = levels[metricType];
  if (!isMetricTypeValid(level)) return false;
  if (globalElementMetricsConfigEnabled && Number.isInteger(context.nowMetricsLevel)) return level >= context.nowMetricsLevel;
  return globalMetricsLevelIsInteger && level >= globalMetricsLevel;
}
//# sourceMappingURL=shouldMark.js.map