/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { timingMetricTypes, timings } from './constants';
import { clearDeferMetrics, getDeferMetrics, logDuration } from '@servicenow/ui-utils';
let debounceMetrics = {}; //Have to do this to avoid circular dependency between utils and metrics packages

let deferMetrics = getDeferMetrics();
export const clearTimingMetrics = () => {
  debounceMetrics = {};
  clearDeferMetrics();
  deferMetrics = getDeferMetrics();
};
export const logTimingMetrics = (type, duration = 0, componentName) => {
  switch (type) {
    case timingMetricTypes.DEFER:
      logDuration(deferMetrics, duration, componentName);
      break;

    case timingMetricTypes.DEBOUNCE:
      logDuration(debounceMetrics, duration, componentName);
      break;
  }
};
export function getCountersHistogram() {
  return {
    [timingMetricTypes.DEFER]: getHistogram(deferMetrics),
    [timingMetricTypes.DEBOUNCE]: getHistogram(debounceMetrics)
  };
}

function getHistogram(metrics) {
  const histogram = {
    [timings.LOWER_BOUNDARY]: {},
    [timings.MIDDLE]: {},
    [timings.UPPER_BOUNDARY]: {}
  };
  Object.keys(metrics).forEach(componentName => {
    const delayTimeArray = metrics[componentName] || [];
    delayTimeArray.forEach(delayTime => {
      if (delayTime <= 50) {
        histogram[timings.LOWER_BOUNDARY][componentName] = (histogram[timings.LOWER_BOUNDARY][componentName] ? histogram[timings.LOWER_BOUNDARY][componentName] : 0) + 1;
      } else if (delayTime > 100) {
        histogram[timings.UPPER_BOUNDARY][componentName] = (histogram[timings.UPPER_BOUNDARY][componentName] ? histogram[timings.UPPER_BOUNDARY][componentName] : 0) + 1;
      } else {
        histogram[timings.MIDDLE][componentName] = (histogram[timings.MIDDLE][componentName] ? histogram[timings.MIDDLE][componentName] : 0) + 1;
      }
    });
  });
  return histogram;
}
//# sourceMappingURL=timingMetrics.js.map